Neko 1.99.6
A portable framework for high-order spectral element flow simulations
Loading...
Searching...
No Matches
gmres_aux.cu
Go to the documentation of this file.
1/*
2 Copyright (c) 2022-2025, The Neko Authors
3 All rights reserved.
4
5 Redistribution and use in source and binary forms, with or without
6 modification, are permitted provided that the following conditions
7 are met:
8
9 * Redistributions of source code must retain the above copyright
10 notice, this list of conditions and the following disclaimer.
11
12 * Redistributions in binary form must reproduce the above
13 copyright notice, this list of conditions and the following
14 disclaimer in the documentation and/or other materials provided
15 with the distribution.
16
17 * Neither the name of the authors nor the names of its
18 contributors may be used to endorse or promote products derived
19 from this software without specific prior written permission.
20
21 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
22 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
23 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
24 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
25 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
26 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
27 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
28 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
29 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
30 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
31 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
32 POSSIBILITY OF SUCH DAMAGE.
33*/
34
35#include <stdio.h>
36#include "gmres_kernel.h"
38#include <device/cuda/check.h>
39#include <device/cuda/buffer.h>
40
41#ifdef HAVE_NVSHMEM
42#include <nvshmem.h>
43#include <nvshmemx.h>
44#endif
45
46extern "C" {
47
50
51#ifdef HAVE_NCCL
54#endif
55
61
62 real cuda_gmres_part2(void *w, void *v, void *h, void * mult, int *j, int *n) {
63
64 const dim3 nthrds(1024, 1, 1);
65 const dim3 nblcks(((*n)+1024 - 1)/ 1024, 1, 1);
66 const int nb = ((*n) + 1024 - 1)/ 1024;
67 const cudaStream_t stream = (cudaStream_t) glb_cmd_queue;
68
72
74 <<<nblcks, nthrds, 0, stream>>>((real *) w, (real **) v,
75 (real *) mult,(real *) h,
76 (real *) gmres_bfd1, *j, *n);
78 reduce_kernel<real><<<1, 1024, 0, stream>>>((real *) gmres_bfd1, nb);
80
81#ifdef HAVE_NCCL
83 DEVICE_NCCL_SUM, stream);
85 cudaMemcpyDeviceToHost, stream));
87#elif HAVE_NVSHMEM
88 if (sizeof(real) == sizeof(float)) {
90 (float *) gmres_bfd1,
91 (float *) gmres_bfd1, 1, stream);
92 }
93 else if (sizeof(real) == sizeof(double)) {
95 (double *) gmres_bfd1,
96 (double *) gmres_bfd1, 1, stream);
97 }
99 cudaMemcpyDeviceToHost, stream));
100 cudaStreamSynchronize(stream);
101#elif HAVE_DEVICE_MPI
102 cudaStreamSynchronize(stream);
104#else
105
107 cudaMemcpyDeviceToHost, stream));
108 cudaStreamSynchronize(stream);
109#endif
110 return gmres_bf1[0];
111 }
112}
void cuda_buffer_reserve(cuda_buffer_t *buf, size_t size)
Definition buffer.cu:50
__global__ void ale_add_kinematics_kernel(const int n, T *__restrict__ wx, T *__restrict__ wy, T *__restrict__ wz, const T *__restrict__ x_ref, const T *__restrict__ y_ref, const T *__restrict__ z_ref, const T *__restrict__ phi, const T *__restrict__ x, const T *__restrict__ y, const T *__restrict__ z, const kinematics_params_t kin_params)
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ w
__global__ void T *__restrict__ T *__restrict__ const T *__restrict__ const T *__restrict__ v
const int j
#define CUDA_BUFFER_INIT_SYMM
Definition buffer.h:63
#define CUDA_CHECK(err)
Definition check.h:6
double real
#define DEVICE_MPI_SUM
void device_mpi_allreduce(void *buf_d, void *buf, int count, int nbytes, int op)
#define DEVICE_NCCL_SUM
void device_nccl_allreduce(void *sbuf_d, void *rbuf_d, int count, int nbytes, int op, void *stream)
cuda_buffer_t gmres_redbuf
Definition gmres_aux.cu:60
real cuda_gmres_part2(void *w, void *v, void *h, void *mult, int *j, int *n)
Definition gmres_aux.cu:62
void * host
Definition buffer.h:52
void * dev
Definition buffer.h:53